==> phoenix-5.2.0-src.tar.gz: phoenix-5.2.0/phoenix-core-client/src/main/java/org/apache/phoenix/schema/types/PTime.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.phoenix.schema.types;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Types;
import java.text.Format;
import java.time.ZoneOffset;

import org.apache.hadoop.hbase.util.Bytes;
import org.apache.phoenix.schema.SortOrder;
import org.apache.phoenix.util.DateUtil;

public class PTime extends PDataType<Time> {

  public static final PTime INSTANCE = new PTime();

  private PTime() {
    super("TIME", Types.TIME, java.sql.Time.class, new PDate.DateCodec(), 10);
  }

  @Override
  public byte[] toBytes(Object object) {
    return PDate.INSTANCE.toBytes(object);
  }

  @Override
  public int toBytes(Object object, byte[] bytes, int offset) {
    return PDate.INSTANCE.toBytes(object, bytes, offset);
  }

  @Override
  public java.sql.Time toObject(byte[] b, int o, int l, PDataType actualType, SortOrder sortOrder,
      Integer maxLength, Integer scale) {
    if (l == 0) {
      return null;
    }
    if (equalsAny(actualType, PTimestamp.INSTANCE, PUnsignedTimestamp.INSTANCE, PDate.INSTANCE,
        PUnsignedDate.INSTANCE, PTime.INSTANCE, PUnsignedTime.INSTANCE, PLong.INSTANCE,
        PUnsignedLong.INSTANCE)) {
      return new java.sql.Time(DateUtil.getCodecFor(actualType).decodeLong(b, o, sortOrder));
    } else if (actualType == PDecimal.INSTANCE) {
      BigDecimal bd = (BigDecimal) actualType.toObject(b, o, l, actualType, sortOrder);
      return new java.sql.Time(bd.longValueExact());
    }
    throwConstraintViolationException(actualType, this);
    return null;
  }

  @Override
  public Object toObject(Object object, PDataType actualType) {
    if (object == null) {
      return null;
    }
    if (equalsAny(actualType, PDate.INSTANCE, PUnsignedDate.INSTANCE)) {
      return new java.sql.Time(((java.util.Date) object).getTime());
    } else if (equalsAny(actualType, PTimestamp.INSTANCE, PUnsignedTimestamp.INSTANCE)) {
      return new java.sql.Time(((java.sql.Timestamp) object).getTime());
    } else if (equalsAny(actualType, PTime.INSTANCE, PUnsignedTime.INSTANCE)) {
      return object;
    } else if (equalsAny(actualType, PLong.INSTANCE, PUnsignedLong.INSTANCE)) {
      return new java.sql.Time((Long) object);
    } else if (actualType == PDecimal.INSTANCE) {
      return new java.sql.Time(((BigDecimal) object).longValueExact());
    } else if (actualType == PVarchar.INSTANCE) {
      return DateUtil.parseTime((String) object);
    }
    return throwConstraintViolationException(actualType, this);
  }

  @Override
  public Object toObject(byte[] bytes, int offset, int length, PDataType actualType,
          SortOrder sortOrder, Integer maxLength, Integer scale, Class jdbcType)
          throws SQLException {
      java.sql.Time sqlTime =
              toObject(bytes, offset, length, actualType, sortOrder, maxLength, scale);
      return timeToClass(sqlTime, actualType, jdbcType);
  }

  Object timeToClass(java.sql.Time sqlTime, PDataType actualType, Class jdbcType)
          throws SQLException {
      if (jdbcType == java.time.LocalTime.class) {
          // FIXME this does a lot of unnecessary computation.
          return java.time.LocalDateTime
                  .ofInstant(java.time.Instant.ofEpochMilli(sqlTime.getTime()), ZoneOffset.UTC)
                  .toLocalTime();
      } else if (jdbcType == java.time.LocalDateTime.class) {
          // This is NOT JDBC compliant
          // We cannot use toInstant(), as that nulls the time fields.
          return java.time.LocalDateTime
                  .ofInstant(java.time.Instant.ofEpochMilli(sqlTime.getTime()), ZoneOffset.UTC);
      } else if (jdbcType == java.time.LocalDate.class) {
          // This is NOT JDBC compliant
          // FIXME this does a lot of unnecessary computation.
          return java.time.LocalDateTime
                  .ofInstant(java.time.Instant.ofEpochMilli(sqlTime.getTime()), ZoneOffset.UTC)
                  .toLocalDate();
      } else if (jdbcType == java.sql.Time.class) {
          return sqlTime;
      } else if (jdbcType == java.sql.Date.class) {
          return new java.sql.Date(sqlTime.getTime());
      } else if (jdbcType == java.sql.Timestamp.class) {
          return new java.sql.Timestamp(sqlTime.getTime());
      } else if (jdbcType == java.util.Date.class) {
          return new java.util.Date(sqlTime.getTime());
      }
      throw newMismatchException(actualType, jdbcType);
  }

  @Override
  public boolean isCastableTo(PDataType targetType) {
    return PDate.INSTANCE.isCastableTo(targetType);
  }

  @Override
  public boolean isCoercibleTo(PDataType targetType) {
    return PDate.INSTANCE.isCoercibleTo(targetType);
  }

  @Override
  public boolean isCoercibleTo(PDataType targetType, Object value) {
    return PDate.INSTANCE.isCoercibleTo(targetType, value);
  }

  @Override
  public boolean isFixedWidth() {
    return true;
  }

  @Override
  public Integer getByteSize() {
    return Bytes.SIZEOF_LONG;
  }

  @Override
  public int compareTo(Object lhs, Object rhs, PDataType rhsType) {
    return PDate.INSTANCE.compareTo(lhs, rhs, rhsType);
  }

  @Override
  public Object toObject(String value) {
    if (value == null || value.length() == 0) {
      return null;
    }
    return DateUtil.parseTime(value);
  }

  @Override
  public boolean isBytesComparableWith(PDataType otherType) {
    return super.isBytesComparableWith(otherType) || otherType == PDate.INSTANCE || otherType == PTimestamp.INSTANCE || otherType == PLong.INSTANCE;
  }

  @Override
  public String toStringLiteral(Object o, Format formatter) {
      if (formatter == null) {
          formatter = DateUtil.DEFAULT_TIME_FORMATTER;
        }
        return "'" + super.toStringLiteral(o, formatter) + "'";
  }

  @Override
  public Object getSampleValue(Integer maxLength, Integer arrayLength) {
    return new java.sql.Time((Long) PLong.INSTANCE.getSampleValue(maxLength, arrayLength));
  }
}

==> phoenix-5.3.0-src.tar.gz: phoenix-5.3.0/phoenix-core-client/src/main/java/org/apache/phoenix/schema/types/PTime.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.phoenix.schema.types;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Types;
import java.text.Format;
import java.time.ZoneOffset;
import org.apache.hadoop.hbase.util.Bytes;
import org.apache.phoenix.schema.SortOrder;
import org.apache.phoenix.util.DateUtil;

public class PTime extends PDataType<Time> {

  public static final PTime INSTANCE = new PTime();

  private PTime() {
    super("TIME", Types.TIME, java.sql.Time.class, new PDate.DateCodec(), 10);
  }

  @Override
  public byte[] toBytes(Object object) {
    return PDate.INSTANCE.toBytes(object);
  }

  @Override
  public int toBytes(Object object, byte[] bytes, int offset) {
    return PDate.INSTANCE.toBytes(object, bytes, offset);
  }

  @Override
  public java.sql.Time toObject(byte[] b, int o, int l, PDataType actualType, SortOrder sortOrder,
    Integer maxLength, Integer scale) {
    if (l == 0) {
      return null;
    }
    if (
      equalsAny(actualType, PTimestamp.INSTANCE, PUnsignedTimestamp.INSTANCE, PDate.INSTANCE,
        PUnsignedDate.INSTANCE, PTime.INSTANCE, PUnsignedTime.INSTANCE, PLong.INSTANCE,
        PUnsignedLong.INSTANCE)
    ) {
      return new java.sql.Time(DateUtil.getCodecFor(actualType).decodeLong(b, o, sortOrder));
    } else if (actualType == PDecimal.INSTANCE) {
      BigDecimal bd = (BigDecimal) actualType.toObject(b, o, l, actualType, sortOrder);
      return new java.sql.Time(bd.longValueExact());
    }
    throwConstraintViolationException(actualType, this);
    return null;
  }

  @Override
  public Object toObject(Object object, PDataType actualType) {
    if (object == null) {
      return null;
    }
    if (equalsAny(actualType, PDate.INSTANCE, PUnsignedDate.INSTANCE)) {
      return new java.sql.Time(((java.util.Date) object).getTime());
    } else if (equalsAny(actualType, PTimestamp.INSTANCE, PUnsignedTimestamp.INSTANCE)) {
      return new java.sql.Time(((java.sql.Timestamp) object).getTime());
    } else if (equalsAny(actualType, PTime.INSTANCE, PUnsignedTime.INSTANCE)) {
      return object;
    } else if (equalsAny(actualType, PLong.INSTANCE, PUnsignedLong.INSTANCE)) {
      return new java.sql.Time((Long) object);
    } else if (actualType == PDecimal.INSTANCE) {
      return new java.sql.Time(((BigDecimal) object).longValueExact());
    } else if (actualType == PVarchar.INSTANCE) {
      return DateUtil.parseTime((String) object);
    }
    return throwConstraintViolationException(actualType, this);
  }

  @Override
  public Object toObject(byte[] bytes, int offset, int length, PDataType actualType,
    SortOrder sortOrder, Integer maxLength, Integer scale, Class jdbcType) throws SQLException {
    java.sql.Time sqlTime =
      toObject(bytes, offset, length, actualType, sortOrder, maxLength, scale);
    return timeToClass(sqlTime, actualType, jdbcType);
  }

  Object timeToClass(java.sql.Time sqlTime, PDataType actualType, Class jdbcType)
    throws SQLException {
    if (jdbcType == java.time.LocalTime.class) {
      // FIXME this does a lot of unnecessary computation.
      return java.time.LocalDateTime
        .ofInstant(java.time.Instant.ofEpochMilli(sqlTime.getTime()), ZoneOffset.UTC).toLocalTime();
    } else if (jdbcType == java.time.LocalDateTime.class) {
      // This is NOT JDBC compliant
      // We cannot use toInstant(), as that nulls the time fields.
      return java.time.LocalDateTime.ofInstant(java.time.Instant.ofEpochMilli(sqlTime.getTime()),
        ZoneOffset.UTC);
    } else if (jdbcType == java.time.LocalDate.class) {
      // This is NOT JDBC compliant
      // FIXME this does a lot of unnecessary computation.
      return java.time.LocalDateTime
        .ofInstant(java.time.Instant.ofEpochMilli(sqlTime.getTime()), ZoneOffset.UTC).toLocalDate();
    } else if (jdbcType == java.sql.Time.class) {
      return sqlTime;
    } else if (jdbcType == java.sql.Date.class) {
      return new java.sql.Date(sqlTime.getTime());
    } else if (jdbcType == java.sql.Timestamp.class) {
      return new java.sql.Timestamp(sqlTime.getTime());
    } else if (jdbcType == java.util.Date.class) {
      return new java.util.Date(sqlTime.getTime());
    }
    throw newMismatchException(actualType, jdbcType);
  }

  @Override
  public boolean isCastableTo(PDataType targetType) {
    return PDate.INSTANCE.isCastableTo(targetType);
  }

  @Override
  public boolean isCoercibleTo(PDataType targetType) {
    return PDate.INSTANCE.isCoercibleTo(targetType);
  }

  @Override
  public boolean isCoercibleTo(PDataType targetType, Object value) {
    return PDate.INSTANCE.isCoercibleTo(targetType, value);
  }

  @Override
  public boolean isFixedWidth() {
    return true;
  }

  @Override
  public Integer getByteSize() {
    return Bytes.SIZEOF_LONG;
  }

  @Override
  public int compareTo(Object lhs, Object rhs, PDataType rhsType) {
    return PDate.INSTANCE.compareTo(lhs, rhs, rhsType);
  }

  @Override
  public Object toObject(String value) {
    if (value == null || value.length() == 0) {
      return null;
    }
    return DateUtil.parseTime(value);
  }

  @Override
  public boolean isBytesComparableWith(PDataType otherType) {
    return super.isBytesComparableWith(otherType) || otherType == PDate.INSTANCE
      || otherType == PTimestamp.INSTANCE || otherType == PLong.INSTANCE;
  }

  @Override
  public String toStringLiteral(Object o, Format formatter) {
    if (formatter == null) {
      formatter = DateUtil.DEFAULT_TIME_FORMATTER;
    }
    return "'" + super.toStringLiteral(o, formatter) + "'";
  }

  @Override
  public Object getSampleValue(Integer maxLength, Integer arrayLength) {
    return new java.sql.Time((Long) PLong.INSTANCE.getSampleValue(maxLength, arrayLength));
  }
}
